% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %
% %
% %              Multi-resolution fusion of Thermal Infrared Hyperspectral and VIS Data for
% %                  the contest of IGARSS 2014
% %    
% %                             
% %           
% %      Copyright notes
% %      Author: Wenzhi Liao IPI, Telin, Ghent University, Belgium
% %      Email: wenzhi.liao@telin.ugent.be, wenzi.liao@gmail.com
% %      Date: 30/10/2014
% %
%       [1] W. Liao, X. Huang, F. Coillie, S. Gautama, A. Pizurica, W. Philips, H. Liu, T. Zhu, 
%           M. Shimoni, G. Moser, and D. Tuia, “Processing of Multiresolution Thermal Hyperspectral and 
%           Digital Color Data: Outcome of the 2014 IEEE GRSS Data Fusion Contest,” IEEE Journal of 
%           Selected Topics in Applied Earth Observations and Remote Sensing, vol. 8, no. 6, pp. 2984-2996, June 2015.
%
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all
close all
p = fileparts(mfilename('fullpath'));
addpath(fullfile(p,'/Quality_Indices'));

%% Read data
load Sim_RGB_HSIAvg30dB;%20dB
%% Contain %%%%%
% HR_RGB: high resolution RGB image
% LR_HSI: low resolution hyperspectral image
% Ref_HSI: reference hyperspectral image
[rows,cols,bands]=size(Ref_HSI);
No_PCs=5;
r=16;eps = 0.01^2;
[Enhanced_HSI,CTime] = GFPCA(LR_HSI, HR_RGB, No_PCs, r, eps);

%% Show image of a single band
band_no = 10;
figure, subplot(1,2,1);imagesc(Ref_HSI(:,:,band_no))
axis image, colormap gray
subplot(1,2,2);imagesc(Enhanced_HSI(:,:,band_no))
axis image, colormap gray 

%%  display some spectrums after fusion
i=1:bands;
a=reshape(Ref_HSI(120,200,:),1,bands);
d=reshape(Enhanced_HSI(120,200,:),1,bands);
figure;plot(i,a,'-k',i,d,'-r')
title('Spectral reflection')

a=reshape(Ref_HSI(147,154,:),1,bands);
d=reshape(Enhanced_HSI(147,154,:),1,bands);
figure;plot(i,a,'-k',i,d,'-r')
title('Spectral reflection')

%% calculate the Q index and some matric
Qavg_orig = Q(Enhanced_HSI,Ref_HSI, 2)
angle_SAM = SAM(Enhanced_HSI,Ref_HSI)
ERGAS = ERGAS(Enhanced_HSI,Ref_HSI,3)

%% show three bands compositions
R=Ref_HSI(:,:,80);
R=255*(R-min(R(:)))/(max(R(:))-min(R(:)));
G=Ref_HSI(:,:,60);
G=255*(G-min(G(:)))/(max(G(:))-min(G(:)));
B=Ref_HSI(:,:,50);
B=255*(B-min(B(:)))/(max(B(:))-min(B(:)));
Ref_RGB=cat(3,R,G,B);
figure;imagesc(uint8(Ref_RGB))

R=LR_HSI(:,:,80);
R=255*(R-min(R(:)))/(max(R(:))-min(R(:)));
G=LR_HSI(:,:,60);
G=255*(G-min(G(:)))/(max(G(:))-min(G(:)));
B=LR_HSI(:,:,50);
B=255*(B-min(B(:)))/(max(B(:))-min(B(:)));
LR_RGB=cat(3,R,G,B);
figure;imagesc(uint8(LR_RGB))


R=Enhanced_HSI(:,:,80);
R=255*(R-min(R(:)))/(max(R(:))-min(R(:)));
G=Enhanced_HSI(:,:,60);
G=255*(G-min(G(:)))/(max(G(:))-min(G(:)));
B=Enhanced_HSI(:,:,50);
B=255*(B-min(B(:)))/(max(B(:))-min(B(:)));
Enhanced_RGB=cat(3,R,G,B);
figure;imagesc(uint8(Enhanced_RGB))

